@@ -22,15 +22,24 @@ THE SOFTWARE.
 package dyld
 
 import (
+	"context"
 	"encoding/json"
 	"fmt"
 	"os"
 	"path/filepath"
+	"slices"
+	"strings"
+	"time"
 
+	"github.com/AlecAivazis/survey/v2"
+	"github.com/AlecAivazis/survey/v2/terminal"
 	"github.com/MakeNowJust/heredoc/v2"
+	"github.com/alecthomas/chroma/v2/quick"
 	"github.com/apex/log"
+	"github.com/blacktop/ipsw/internal/ai"
 	"github.com/blacktop/ipsw/pkg/disass"
 	"github.com/blacktop/ipsw/pkg/dyld"
+	"github.com/briandowns/spinner"
 	"github.com/fatih/color"
 	"github.com/pkg/errors"
 	"github.com/spf13/cobra"
@@ -45,7 +54,17 @@ func init() {
 	DisassCmd.Flags().String("symbol-image", "", "Dylib to search for symbol (speeds up symbol lookup)")
 	DisassCmd.Flags().Uint64P("vaddr", "a", 0, "Virtual address to start disassembling")
 	DisassCmd.Flags().Uint64P("count", "c", 0, "Number of instructions to disassemble")
+	DisassCmd.Flags().Bool("dylibs", false, "Analyze all dylibs loaded by the image as well (could improve accuracy)")
 	DisassCmd.Flags().BoolP("demangle", "d", false, "Demangle symbol names")
+	DisassCmd.Flags().BoolP("dec", "D", false, "Decompile assembly")
+	DisassCmd.Flags().String("dec-model", "", "LLM model to use for decompilation")
+	DisassCmd.Flags().String("dec-lang", "", "Language to decompile to (C, ObjC or Swift)")
+	DisassCmd.Flags().Float64("dec-temp", 0.2, "LLM temperature for decompilation")
+	DisassCmd.Flags().Float64("dec-top-p", 0.1, "LLM top_p for decompilation")
+	DisassCmd.Flags().String("llm", "copilot", "LLM provider to use for decompilation (ollama, copilot, etc.)")
+	DisassCmd.RegisterFlagCompletionFunc("llm", func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
+		return ai.Providers, cobra.ShellCompDirectiveDefault
+	})
 	DisassCmd.Flags().BoolP("json", "j", false, "Output as JSON")
 	DisassCmd.Flags().BoolP("quiet", "q", false, "Do NOT markup analysis (Faster)")
 	DisassCmd.Flags().Bool("force", false, "Continue to disassemble even if there are analysis errors")
@@ -58,7 +77,14 @@ func init() {
 	viper.BindPFlag("dyld.disass.symbol-image", DisassCmd.Flags().Lookup("symbol-image"))
 	viper.BindPFlag("dyld.disass.vaddr", DisassCmd.Flags().Lookup("vaddr"))
 	viper.BindPFlag("dyld.disass.count", DisassCmd.Flags().Lookup("count"))
+	viper.BindPFlag("dyld.disass.dylibs", DisassCmd.Flags().Lookup("dylibs"))
 	viper.BindPFlag("dyld.disass.demangle", DisassCmd.Flags().Lookup("demangle"))
+	viper.BindPFlag("dyld.disass.dec", DisassCmd.Flags().Lookup("dec"))
+	viper.BindPFlag("dyld.disass.dec-model", DisassCmd.Flags().Lookup("dec-model"))
+	viper.BindPFlag("dyld.disass.dec-lang", DisassCmd.Flags().Lookup("dec-lang"))
+	viper.BindPFlag("dyld.disass.dec-temp", DisassCmd.Flags().Lookup("dec-temp"))
+	viper.BindPFlag("dyld.disass.dec-top-p", DisassCmd.Flags().Lookup("dec-top-p"))
+	viper.BindPFlag("dyld.disass.llm", DisassCmd.Flags().Lookup("llm"))
 	viper.BindPFlag("dyld.disass.json", DisassCmd.Flags().Lookup("json"))
 	viper.BindPFlag("dyld.disass.quiet", DisassCmd.Flags().Lookup("quiet"))
 	viper.BindPFlag("dyld.disass.force", DisassCmd.Flags().Lookup("force"))
@@ -87,7 +113,9 @@ var DisassCmd = &cobra.Command{
 		# Disassemble a function at a virtual address in dyld_shared_cache and demangle symbol names
 		❯ ipsw dsc disass DSC --vaddr 0x1b19d6940 --demangle
 		# Disassemble a function at a virtual address in dyld_shared_cache and do NOT markup analysis (Faster)
-		❯ ipsw dsc disass DSC --vaddr 0x1b19d6940 --quiet`),
+		❯ ipsw dsc disass DSC --vaddr 0x1b19d6940 --quiet
+		# Decompile a function at a virtual address in dyld_shared_cache (via GitHub Copilot)
+		❯ ipsw dsc disass DSC --vaddr 0x1b19d6940 --dec --dec-model "Claude 3.7 Sonnet"`),
 	Args: cobra.ExactArgs(1),
 	ValidArgsFunction: func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		return getDSCs(toComplete), cobra.ShellCompDirectiveDefault
@@ -112,6 +140,7 @@ var DisassCmd = &cobra.Command{
 		startAddr := viper.GetUint64("dyld.disass.vaddr")
 		instructions := viper.GetUint64("dyld.disass.count")
 
+		decompile := viper.GetBool("dyld.disass.dec")
 		demangleFlag := viper.GetBool("dyld.disass.demangle")
 		asJSON := viper.GetBool("dyld.disass.json")
 		quiet := viper.GetBool("dyld.disass.quiet")
@@ -122,6 +151,11 @@ var DisassCmd = &cobra.Command{
 		if len(symbolImageName) > 0 && len(symbolName) == 0 {
 			return fmt.Errorf("you must also supply a --symbol with --symbol-image flag")
 		}
+		if viper.IsSet("dyld.disass.llm") {
+			if !slices.Contains(ai.Providers, viper.GetString("dyld.disass.llm")) {
+				return fmt.Errorf("invalid LLM provider '%s', must be one of: %s", viper.GetString("dyld.disass.llm"), strings.Join(ai.Providers, ", "))
+			}
+		}
 
 		dscPath := filepath.Clean(args[0])
 
@@ -154,7 +188,7 @@ var DisassCmd = &cobra.Command{
 			return nil
 		}
 
-		if !quiet && len(symbolName) > 0 {
+		if !quiet || len(symbolName) > 0 {
 			if len(cacheFile) == 0 {
 				cacheFile = dscPath + ".a2s"
 			}
@@ -214,10 +248,12 @@ var DisassCmd = &cobra.Command{
 						if err := engine.Triage(); err != nil {
 							return fmt.Errorf("first pass triage failed: %v", err)
 						}
-						for _, img := range engine.Dylibs() {
-							if err := img.Analyze(); err != nil {
-								if !viper.GetBool("dyld.disass.force") {
-									return fmt.Errorf("failed to analyze image %s: %v (use --force to continue anyway)", filepath.Base(img.Name), err)
+						if viper.GetBool("dyld.disass.dylibs") {
+							for _, img := range engine.Dylibs() {
+								if err := img.Analyze(); err != nil {
+									if !viper.GetBool("dyld.disass.force") {
+										return fmt.Errorf("failed to analyze image %s: %v (use --force to continue anyway)", filepath.Base(img.Name), err)
+									}
 								}
 							}
 						}
@@ -233,7 +269,57 @@ var DisassCmd = &cobra.Command{
 					//***************
 					//* DISASSEMBLE *
 					//***************
-					disass.Disassemble(engine)
+					asm := disass.Disassemble(engine)
+					if decompile {
+						promptFmt, lexer, err := disass.GetPrompt(asm, viper.GetString("dyld.disass.dec-lang"))
+						if err != nil {
+							return fmt.Errorf("failed to get prompt format string and syntax highlight lexer: %v", err)
+						}
+						llm, err := ai.NewAI(context.Background(), &ai.Config{
+							Provider:    viper.GetString("dyld.disass.llm"),
+							Prompt:      fmt.Sprintf(promptFmt, asm),
+							Model:       viper.GetString("dyld.disass.dec-model"),
+							Temperature: viper.GetFloat64("dyld.disass.dec-temp"),
+							TopP:        viper.GetFloat64("dyld.disass.dec-top-p"),
+							Stream:      true,
+						})
+						if err != nil {
+							return fmt.Errorf("failed to create llm client: %v", err)
+						}
+						if !viper.IsSet("dyld.disass.dec-model") {
+							var choice string
+							prompt := &survey.Select{
+								Message:  "Select model to use:",
+								Options:  llm.Models(),
+								PageSize: 10,
+							}
+							if err := survey.AskOne(prompt, &choice); err == terminal.InterruptErr {
+								log.Warn("Exiting...")
+								return nil
+							}
+							if err := llm.SetModel(choice); err != nil {
+								return fmt.Errorf("failed to set llm model: %v", err)
+							}
+						}
+						s := spinner.New(spinner.CharSets[38], 100*time.Millisecond)
+						s.Prefix = color.BlueString("   • Decompiling... ")
+						s.Start()
+
+						decmp, err := llm.Chat()
+						s.Stop()
+						if err != nil {
+							return fmt.Errorf("failed to decompile via llm: %v", err)
+						}
+						if viper.GetBool("color") && !viper.GetBool("no-color") {
+							if err := quick.Highlight(os.Stdout, "\n"+string(decmp)+"\n", lexer, "terminal256", "nord"); err != nil {
+								return err
+							}
+						} else {
+							fmt.Println(string(decmp))
+						}
+					} else {
+						fmt.Println(asm)
+					}
 				}
 			}
 
@@ -308,10 +394,12 @@ var DisassCmd = &cobra.Command{
 						if err := engine.Triage(); err != nil {
 							return fmt.Errorf("first pass triage failed: %v", err)
 						}
-						for _, img := range engine.Dylibs() {
-							if err := img.Analyze(); err != nil {
-								if !viper.GetBool("dyld.disass.force") {
-									return fmt.Errorf("failed to analyze image %s: %v (use --force to continue anyway)", filepath.Base(img.Name), err)
+						if viper.GetBool("dyld.disass.dylibs") {
+							for _, img := range engine.Dylibs() {
+								if err := img.Analyze(); err != nil {
+									if !viper.GetBool("dyld.disass.force") {
+										return fmt.Errorf("failed to analyze image %s: %v (use --force to continue anyway)", filepath.Base(img.Name), err)
+									}
 								}
 							}
 						}
@@ -328,7 +416,57 @@ var DisassCmd = &cobra.Command{
 					//***************
 					//* DISASSEMBLE *
 					//***************
-					disass.Disassemble(engine)
+					asm := disass.Disassemble(engine)
+					if decompile {
+						promptFmt, lexer, err := disass.GetPrompt(asm, viper.GetString("dyld.disass.dec-lang"))
+						if err != nil {
+							return fmt.Errorf("failed to get prompt format string and syntax highlight lexer: %v", err)
+						}
+						llm, err := ai.NewAI(context.Background(), &ai.Config{
+							Provider:    viper.GetString("dyld.disass.llm"),
+							Prompt:      fmt.Sprintf(promptFmt, asm),
+							Model:       viper.GetString("dyld.disass.dec-model"),
+							Temperature: viper.GetFloat64("dyld.disass.dec-temp"),
+							TopP:        viper.GetFloat64("dyld.disass.dec-top-p"),
+							Stream:      true,
+						})
+						if err != nil {
+							return fmt.Errorf("failed to create llm client: %v", err)
+						}
+						if !viper.IsSet("dyld.disass.dec-model") {
+							var choice string
+							prompt := &survey.Select{
+								Message:  "Select model to use:",
+								Options:  llm.Models(),
+								PageSize: 10,
+							}
+							if err := survey.AskOne(prompt, &choice); err == terminal.InterruptErr {
+								log.Warn("Exiting...")
+								return nil
+							}
+							if err := llm.SetModel(choice); err != nil {
+								return fmt.Errorf("failed to set llm model: %v", err)
+							}
+						}
+						s := spinner.New(spinner.CharSets[38], 100*time.Millisecond)
+						s.Prefix = color.BlueString("   • Decompiling... ")
+						s.Start()
+
+						decmp, err := llm.Chat()
+						s.Stop()
+						if err != nil {
+							return fmt.Errorf("failed to decompile via llm: %v", err)
+						}
+						if viper.GetBool("color") && !viper.GetBool("no-color") {
+							if err := quick.Highlight(os.Stdout, "\n"+string(decmp)+"\n", lexer, "terminal256", "nord"); err != nil {
+								return err
+							}
+						} else {
+							fmt.Println(string(decmp))
+						}
+					} else {
+						fmt.Println(asm)
+					}
 				}
 			} else { // SYMBOL or VAADDR
 				/*
@@ -348,7 +486,7 @@ var DisassCmd = &cobra.Command{
 						if !quiet {
 							if err := image.Analyze(); err != nil {
 								if !viper.GetBool("dyld.disass.force") {
-									return fmt.Errorf("failed to analyze image %s: %v", filepath.Base(image.Name), err)
+									return fmt.Errorf("failed to analyze image %s: %v (use --force to continue anyway)", filepath.Base(image.Name), err)
 								}
 							}
 						}
@@ -420,10 +558,12 @@ var DisassCmd = &cobra.Command{
 					if err := engine.Triage(); err != nil {
 						return fmt.Errorf("first pass triage failed: %v (use --force to continue anyway)", err)
 					}
-					for _, img := range engine.Dylibs() {
-						if err := img.Analyze(); err != nil {
-							if !viper.GetBool("dyld.disass.force") {
-								return fmt.Errorf("failed to analyze image %s: %v (use --force to continue anyway)", filepath.Base(img.Name), err)
+					if viper.GetBool("dyld.disass.dylibs") {
+						for _, img := range engine.Dylibs() {
+							if err := img.Analyze(); err != nil {
+								if !viper.GetBool("dyld.disass.force") {
+									return fmt.Errorf("failed to analyze image %s: %v (use --force to continue anyway)", filepath.Base(img.Name), err)
+								}
 							}
 						}
 					}
@@ -440,7 +580,57 @@ var DisassCmd = &cobra.Command{
 				//***************
 				//* DISASSEMBLE *
 				//***************
-				disass.Disassemble(engine)
+				asm := disass.Disassemble(engine)
+				if decompile {
+					promptFmt, lexer, err := disass.GetPrompt(asm, viper.GetString("dyld.disass.dec-lang"))
+					if err != nil {
+						return fmt.Errorf("failed to get prompt format string and syntax highlight lexer: %v", err)
+					}
+					llm, err := ai.NewAI(context.Background(), &ai.Config{
+						Provider:    viper.GetString("dyld.disass.llm"),
+						Prompt:      fmt.Sprintf(promptFmt, asm),
+						Model:       viper.GetString("dyld.disass.dec-model"),
+						Temperature: viper.GetFloat64("dyld.disass.dec-temp"),
+						TopP:        viper.GetFloat64("dyld.disass.dec-top-p"),
+						Stream:      true,
+					})
+					if err != nil {
+						return fmt.Errorf("failed to create llm client: %v", err)
+					}
+					if !viper.IsSet("dyld.disass.dec-model") {
+						var choice string
+						prompt := &survey.Select{
+							Message:  "Select model to use:",
+							Options:  llm.Models(),
+							PageSize: 10,
+						}
+						if err := survey.AskOne(prompt, &choice); err == terminal.InterruptErr {
+							log.Warn("Exiting...")
+							return nil
+						}
+						if err := llm.SetModel(choice); err != nil {
+							return fmt.Errorf("failed to set llm model: %v", err)
+						}
+					}
+					s := spinner.New(spinner.CharSets[38], 100*time.Millisecond)
+					s.Prefix = color.BlueString("   • Decompiling... ")
+					s.Start()
+
+					decmp, err := llm.Chat()
+					s.Stop()
+					if err != nil {
+						return fmt.Errorf("failed to decompile via llm: %v", err)
+					}
+					if viper.GetBool("color") && !viper.GetBool("no-color") {
+						if err := quick.Highlight(os.Stdout, "\n"+string(decmp)+"\n", lexer, "terminal256", "nord"); err != nil {
+							return err
+						}
+					} else {
+						fmt.Println(string(decmp))
+					}
+				} else {
+					fmt.Println(asm)
+				}
 			}
 		}
 
