@@ -23,19 +23,23 @@ package macho
 
 import (
 	"context"
-	"encoding/gob"
 	"fmt"
-	"maps"
 	"os"
 	"path/filepath"
+	"slices"
 	"strings"
+	"time"
 
 	"github.com/AlecAivazis/survey/v2"
+	"github.com/AlecAivazis/survey/v2/terminal"
+	"github.com/alecthomas/chroma/v2/quick"
 	"github.com/apex/log"
 	"github.com/blacktop/go-macho"
 	"github.com/blacktop/go-macho/types"
+	"github.com/blacktop/ipsw/internal/ai"
 	"github.com/blacktop/ipsw/internal/magic"
 	"github.com/blacktop/ipsw/pkg/disass"
+	"github.com/briandowns/spinner"
 	"github.com/caarlos0/ctrlc"
 	"github.com/fatih/color"
 	"github.com/pkg/errors"
@@ -53,6 +57,15 @@ func init() {
 	machoDisassCmd.Flags().Uint64P("off", "o", 0, "File offset to start disassembling")
 	machoDisassCmd.Flags().Uint64P("count", "c", 0, "Number of instructions to disassemble")
 	machoDisassCmd.Flags().BoolP("demangle", "d", false, "Demangle symbol names")
+	machoDisassCmd.Flags().BoolP("dec", "D", false, "Decompile assembly")
+	machoDisassCmd.Flags().String("dec-model", "", "LLM model to use for decompilation")
+	machoDisassCmd.Flags().String("dec-lang", "", "Language to decompile to (C, ObjC or Swift)")
+	machoDisassCmd.Flags().Float64("dec-temp", 0.2, "LLM temperature for decompilation")
+	machoDisassCmd.Flags().Float64("dec-top-p", 0.1, "LLM top_p for decompilation")
+	machoDisassCmd.Flags().String("llm", "", "LLM provider to use for decompilation (ollama, copilot, etc.)")
+	machoDisassCmd.RegisterFlagCompletionFunc("llm", func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
+		return ai.Providers, cobra.ShellCompDirectiveDefault
+	})
 	machoDisassCmd.Flags().BoolP("json", "j", false, "Output as JSON")
 	machoDisassCmd.Flags().BoolP("quiet", "q", false, "Do NOT markup analysis (Faster)")
 	machoDisassCmd.Flags().Bool("force", false, "Continue to disassemble even if there are analysis errors")
@@ -71,6 +84,12 @@ func init() {
 	viper.BindPFlag("macho.disass.off", machoDisassCmd.Flags().Lookup("off"))
 	viper.BindPFlag("macho.disass.count", machoDisassCmd.Flags().Lookup("count"))
 	viper.BindPFlag("macho.disass.demangle", machoDisassCmd.Flags().Lookup("demangle"))
+	viper.BindPFlag("macho.disass.dec", machoDisassCmd.Flags().Lookup("dec"))
+	viper.BindPFlag("macho.disass.dec-model", machoDisassCmd.Flags().Lookup("dec-model"))
+	viper.BindPFlag("macho.disass.dec-lang", machoDisassCmd.Flags().Lookup("dec-lang"))
+	viper.BindPFlag("macho.disass.dec-temp", machoDisassCmd.Flags().Lookup("dec-temp"))
+	viper.BindPFlag("macho.disass.dec-top-p", machoDisassCmd.Flags().Lookup("dec-top-p"))
+	viper.BindPFlag("macho.disass.llm", machoDisassCmd.Flags().Lookup("llm"))
 	viper.BindPFlag("macho.disass.json", machoDisassCmd.Flags().Lookup("json"))
 	viper.BindPFlag("macho.disass.quiet", machoDisassCmd.Flags().Lookup("quiet"))
 	viper.BindPFlag("macho.disass.force", machoDisassCmd.Flags().Lookup("force"))
@@ -112,6 +131,7 @@ var machoDisassCmd = &cobra.Command{
 		instructions := viper.GetUint64("macho.disass.count")
 		segmentSection := viper.GetString("macho.disass.section")
 
+		decompile := viper.GetBool("macho.disass.dec")
 		demangleFlag := viper.GetBool("macho.disass.demangle")
 		asJSON := viper.GetBool("macho.disass.json")
 		quiet := viper.GetBool("macho.disass.quiet")
@@ -132,6 +152,11 @@ var machoDisassCmd = &cobra.Command{
 		} else if viper.GetBool("macho.disass.all-fileset-entries") && len(segmentSection) == 0 {
 			log.Warn("you probably want to add --section '__TEXT_EXEC.__text'; as the NEW MH_FILESET entries don't ALL have LC_FUNCTION_STARTS (iOS18 added LC_FUNCTION_STARTS to all KEXTs ❤️)")
 		}
+		if viper.IsSet("macho.disass.llm") {
+			if !slices.Contains(ai.Providers, viper.GetString("macho.disass.llm")) {
+				return fmt.Errorf("invalid LLM provider '%s', must be one of: %s", viper.GetString("macho.disass.llm"), strings.Join(ai.Providers, ", "))
+			}
+		}
 
 		machoPath := filepath.Clean(args[0])
 
@@ -139,10 +164,6 @@ var machoDisassCmd = &cobra.Command{
 			return fmt.Errorf("failed to detect file type: %v", err)
 		}
 
-		// symbol caches
-		symbolMap := make(map[uint64]string)
-		dSymMap := make(map[uint64]string)
-
 		fat, err := macho.OpenFat(machoPath)
 		if err != nil && err != macho.ErrNotFat {
 			log.Fatal(err.Error())
@@ -238,63 +259,6 @@ var machoDisassCmd = &cobra.Command{
 					if len(cacheFile) == 0 {
 						cacheFile = machoPath + ".a2s"
 					}
-					if _, err := os.Stat(cacheFile); errors.Is(err, os.ErrNotExist) {
-						if _, err := os.Create(cacheFile); err != nil {
-							return fmt.Errorf("failed to create address-to-symbol cache file %s: %v", cacheFile, err)
-						}
-						// if dSYM file exists, load symbols from it
-						dsym := filepath.Join(machoPath+".dSYM", "Contents/Resources/DWARF", filepath.Base(machoPath))
-						if _, err := os.Stat(dsym); err == nil {
-							log.Info("Detected dSYM file, using it for symbolication")
-							dm, err := macho.Open(dsym)
-							if err != nil {
-								log.Errorf("failed to open dSYM file for symbolication: %v", err)
-							} else {
-								for _, sym := range dm.Symtab.Syms {
-									if sym.Name != "" {
-										dSymMap[sym.Value] = sym.Name
-									}
-								}
-								// If the dSYM file has a symbol map, use it for the output
-								if len(dSymMap) > 0 {
-									log.Infof("Using dSYM symbol map with %d symbols", len(dSymMap))
-								} else {
-									log.Warn("No symbols found in dSYM file")
-								}
-							}
-						}
-					} else {
-						log.Infof("Loading symbol cache file...")
-						if f, err := os.Open(cacheFile); err != nil {
-							return fmt.Errorf("failed to open address-to-symbol cache file %s: %v", cacheFile, err)
-						} else {
-							if err := gob.NewDecoder(f).Decode(&symbolMap); err != nil {
-								yes := false
-								if viper.GetBool("macho.disass.replace") {
-									yes = true
-								} else {
-									log.Errorf("address-to-symbol cache file is corrupt: %v", err)
-									prompt := &survey.Confirm{
-										Message: fmt.Sprintf("Recreate %s. Continue?", cacheFile),
-										Default: true,
-									}
-									survey.AskOne(prompt, &yes)
-								}
-								if yes {
-									f.Close()
-									if err := os.Remove(cacheFile); err != nil {
-										return fmt.Errorf("failed to remove address-to-symbol cache file %s: %v", cacheFile, err)
-									}
-									if _, err := os.Create(cacheFile); err != nil {
-										return fmt.Errorf("failed to create address-to-symbol cache file %s: %v", cacheFile, err)
-									}
-								} else {
-									return nil
-								}
-							}
-							f.Close()
-						}
-					}
 				}
 
 				if allFuncs && len(segmentSection) == 0 {
@@ -305,7 +269,7 @@ var machoDisassCmd = &cobra.Command{
 							continue
 						}
 
-						engine = disass.NewMachoDisass(m, &symbolMap, &disass.Config{
+						engine = disass.NewMachoDisass(m, &disass.Config{
 							Data:         data,
 							StartAddress: fn.StartAddr,
 							Middle:       0,
@@ -319,17 +283,19 @@ var machoDisassCmd = &cobra.Command{
 						//* First pass ANALYSIS *
 						//***********************
 						if !quiet {
+							if err := engine.OpenOrCreateSymMap(cacheFile, machoPath, viper.GetBool("macho.disass.replace")); err != nil {
+								return fmt.Errorf("failed to open or create symbol map: %v", err)
+							}
 							if err := engine.Triage(); err != nil {
 								return fmt.Errorf("first pass triage failed: %v", err)
 							}
-							if len(symbolMap) == 0 {
+							if engine.EmptySymMap() {
 								if err := engine.Analyze(); err != nil {
 									if !viper.GetBool("macho.disass.force") {
 										return fmt.Errorf("MachO analysis failed: %v (use --force to continue anyway)", err)
 									}
 								}
 							}
-							maps.Copy(symbolMap, dSymMap) // merge dSYM symbols into symbolMap
 							if err := engine.SaveAddrToSymMap(cacheFile); err != nil {
 								log.Errorf("failed to save symbol map: %v", err)
 							}
@@ -337,7 +303,57 @@ var machoDisassCmd = &cobra.Command{
 						//***************
 						//* DISASSEMBLE *
 						//***************
-						disass.Disassemble(engine)
+						asm := disass.Disassemble(engine)
+						if decompile {
+							promptFmt, lexer, err := disass.GetPrompt(asm, viper.GetString("macho.disass.dec-lang"))
+							if err != nil {
+								return fmt.Errorf("failed to get prompt format string and syntax highlight lexer: %v", err)
+							}
+							llm, err := ai.NewAI(context.Background(), &ai.Config{
+								Provider:    viper.GetString("macho.disass.llm"),
+								Prompt:      fmt.Sprintf(promptFmt, asm),
+								Model:       viper.GetString("macho.disass.dec-model"),
+								Temperature: viper.GetFloat64("macho.disass.dec-temp"),
+								TopP:        viper.GetFloat64("macho.disass.dec-top-p"),
+								Stream:      true,
+							})
+							if err != nil {
+								return fmt.Errorf("failed to create llm client: %v", err)
+							}
+							if !viper.IsSet("macho.disass.dec-model") {
+								var choice string
+								prompt := &survey.Select{
+									Message:  "Select model to use:",
+									Options:  llm.Models(),
+									PageSize: 10,
+								}
+								if err := survey.AskOne(prompt, &choice); err == terminal.InterruptErr {
+									log.Warn("Exiting...")
+									return nil
+								}
+								if err := llm.SetModel(choice); err != nil {
+									return fmt.Errorf("failed to set llm model: %v", err)
+								}
+							}
+							s := spinner.New(spinner.CharSets[38], 100*time.Millisecond)
+							s.Prefix = color.BlueString("   • Decompiling... ")
+							s.Start()
+
+							decmp, err := llm.Chat()
+							s.Stop()
+							if err != nil {
+								return fmt.Errorf("failed to decompile via llm: %v", err)
+							}
+							if viper.GetBool("color") && !viper.GetBool("no-color") {
+								if err := quick.Highlight(os.Stdout, "\n"+string(decmp)+"\n", lexer, "terminal256", "nord"); err != nil {
+									return err
+								}
+							} else {
+								fmt.Println(string(decmp))
+							}
+						} else {
+							fmt.Println(asm)
+						}
 					}
 				} else {
 					if len(symbolName) > 0 {
@@ -406,7 +422,7 @@ var machoDisassCmd = &cobra.Command{
 						log.Fatal("failed to disassemble")
 					}
 
-					engine = disass.NewMachoDisass(m, &symbolMap, &disass.Config{
+					engine = disass.NewMachoDisass(m, &disass.Config{
 						Data:         data,
 						StartAddress: startAddr,
 						Middle:       middleAddr,
@@ -420,17 +436,22 @@ var machoDisassCmd = &cobra.Command{
 					//* First pass ANALYSIS *
 					//***********************
 					if !quiet {
+						if err := engine.OpenOrCreateSymMap(cacheFile, machoPath, viper.GetBool("macho.disass.replace")); err != nil {
+							return fmt.Errorf("failed to open or create symbol map: %v", err)
+						}
 						if err := engine.Triage(); err != nil {
 							return fmt.Errorf("first pass triage failed: %v", err)
 						}
-						if len(symbolMap) == 0 {
-							if err := engine.Analyze(); err != nil {
-								if !viper.GetBool("macho.disass.force") {
-									return fmt.Errorf("MachO analysis failed: %v (use --force to continue anyway)", err)
-								}
+						// if engine.EmptySymMap() {
+						if err := engine.Analyze(); err != nil {
+							if !viper.GetBool("macho.disass.force") {
+								return fmt.Errorf("MachO analysis failed: %v (use --force to continue anyway)", err)
 							}
 						}
-						maps.Copy(symbolMap, dSymMap) // merge dSYM symbols into symbolMap
+						// }
+						if entryStart {
+							engine.SetStartSym(startAddr)
+						}
 						if err := engine.SaveAddrToSymMap(cacheFile); err != nil {
 							log.Errorf("failed to save symbol map: %v", err)
 						}
@@ -438,7 +459,57 @@ var machoDisassCmd = &cobra.Command{
 					//***************
 					//* DISASSEMBLE *
 					//***************
-					disass.Disassemble(engine)
+					asm := disass.Disassemble(engine)
+					if decompile {
+						promptFmt, lexer, err := disass.GetPrompt(asm, viper.GetString("macho.disass.dec-lang"))
+						if err != nil {
+							return fmt.Errorf("failed to get prompt format string and syntax highlight lexer: %v", err)
+						}
+						llm, err := ai.NewAI(context.Background(), &ai.Config{
+							Provider:    viper.GetString("macho.disass.llm"),
+							Prompt:      fmt.Sprintf(promptFmt, asm),
+							Model:       viper.GetString("macho.disass.dec-model"),
+							Temperature: viper.GetFloat64("macho.disass.dec-temp"),
+							TopP:        viper.GetFloat64("macho.disass.dec-top-p"),
+							Stream:      true,
+						})
+						if err != nil {
+							return fmt.Errorf("failed to create llm client: %v", err)
+						}
+						if !viper.IsSet("macho.disass.dec-model") {
+							var choice string
+							prompt := &survey.Select{
+								Message:  "Select model to use:",
+								Options:  llm.Models(),
+								PageSize: 10,
+							}
+							if err := survey.AskOne(prompt, &choice); err == terminal.InterruptErr {
+								log.Warn("Exiting...")
+								return nil
+							}
+							if err := llm.SetModel(choice); err != nil {
+								return fmt.Errorf("failed to set llm model: %v", err)
+							}
+						}
+						s := spinner.New(spinner.CharSets[38], 100*time.Millisecond)
+						s.Prefix = color.BlueString("   • Decompiling... ")
+						s.Start()
+
+						decmp, err := llm.Chat()
+						s.Stop()
+						if err != nil {
+							return fmt.Errorf("failed to decompile via llm: %v", err)
+						}
+						if viper.GetBool("color") && !viper.GetBool("no-color") {
+							if err := quick.Highlight(os.Stdout, "\n"+string(decmp)+"\n", lexer, "terminal256", "nord"); err != nil {
+								return err
+							}
+						} else {
+							fmt.Println(string(decmp))
+						}
+					} else {
+						fmt.Println(asm)
+					}
 				}
 			}
 			return nil
