@@ -6,11 +6,14 @@ import (
 	"encoding/binary"
 	"fmt"
 	"io"
+	"maps"
 	"math/bits"
 	"os"
 	"path/filepath"
 	"strings"
 
+	"github.com/AlecAivazis/survey/v2"
+	"github.com/AlecAivazis/survey/v2/terminal"
 	"github.com/apex/log"
 	"github.com/blacktop/go-macho/pkg/codesign"
 	"github.com/blacktop/go-macho/pkg/trie"
@@ -488,7 +491,7 @@ func (f *File) parseCache(r io.ReaderAt, uuid mtypes.UUID) error {
 
 		sr.Seek(int64(f.Headers[uuid].LocalSymbolsOffset+uint64(f.LocalSymInfo.EntriesOffset)), io.SeekStart)
 
-		for i := 0; i < int(f.LocalSymInfo.EntriesCount); i++ {
+		for i := range int(f.LocalSymInfo.EntriesCount) {
 			// if err := binary.Read(sr, f.ByteOrder, &f.Images[i].CacheLocalSymbolsEntry); err != nil {
 			// 	return nil, err
 			// }
@@ -757,9 +760,7 @@ func (f *File) ParseStubIslands() error {
 			if err != nil {
 				return err
 			}
-			for k, v := range stubs {
-				f.islandStubs[k] = v
-			}
+			maps.Copy(f.islandStubs, stubs)
 		}
 	}
 	return nil
@@ -1763,7 +1764,16 @@ func (f *File) Image(name string) (*CacheImage, error) {
 		for _, m := range matches {
 			names = append(names, m.Name)
 		}
-		return nil, fmt.Errorf("multiple images found for %s (please supply FULL path):\n\t- %s", name, strings.Join(names, "\n\t- "))
+		var choice string
+		prompt := &survey.Select{
+			Message: "Multiple images found for " + name + ", please select one:",
+			Options: names,
+		}
+		if err := survey.AskOne(prompt, &choice); err == terminal.InterruptErr {
+			log.Warn("Exiting...")
+			return nil, fmt.Errorf("multiple images found for %s (please supply FULL path):\n\t- %s", name, strings.Join(names, "\n\t- "))
+		}
+		return f.Image(choice)
 	}
 	return nil, fmt.Errorf("image %s not found in cache", name)
 }
